﻿package 
{
	import flash.display.Sprite;
	import flash.display.Stage;
	
	public class FlashBall {
		public var x:Number;
		public var y:Number;
		public var vx:Number;
		public var vy:Number;
		public var asset:Sprite;
		public var rotation:Number;
		public var alpha:Number;
		
		// Some of these variables were exposed as public to support the option
		// for inlining the move and doCollide methods. 
		public var diameter:Number;
		public var diamSquared:Number;
		public var leftBoundary:Number;
		public var topBoundary:Number;
		public var rightBoundary:Number;
		public var bottomBoundary:Number;
		
		private var _radius:Number;
		private var _elasticity:Number;
		private var _ballElasticity:Number;
		private var _maxSpeed:Number;
		
		private var _leftWall:Number;
		private var _topWall:Number;
		private var _rightWall:Number;
		private var _bottomWall:Number;
		
		private var _boundaryHeight:Number;
		private var _boundaryWidth:Number;
		
	
		public function FlashBall($definition:Object)	
		{
			_elasticity = 1.05;//-0.2;
			_ballElasticity = .99;
			_maxSpeed = 2.0; 
			rotation = 0;
			alpha = 1;

			asset = null;

			_radius = $definition.radius;
			diameter = 2 * _radius; 
			diamSquared = diameter * diameter; // diameter squared for faster distance comparisons (no square root required)

			_leftWall = $definition.leftWall; 
			_topWall = $definition.topWall;
			_rightWall = $definition.rightWall; 
			_bottomWall = $definition.bottomWall;
			
			setBoundaries($definition.stageWidth, $definition.stageHeight);
			
			resetPosition();
		}
		
		public function resetPosition():void
		{
			x = int(leftBoundary + _boundaryWidth * Math.random() ); 
			y = int(topBoundary + _boundaryHeight * Math.random() );
			vx = - _maxSpeed + 2 * ( _maxSpeed * Math.random() );
			vy = - _maxSpeed + 2 * ( _maxSpeed * Math.random() );
		}
		
		public function setBoundaries($stageWidth:int, $stageHeight:int):void
		{
			leftBoundary = _leftWall + _radius;
			topBoundary = _topWall + _radius;
			rightBoundary = $stageWidth - _rightWall - _radius;				
			bottomBoundary = $stageHeight - _bottomWall - _radius;
			
			_boundaryHeight = bottomBoundary - topBoundary;
			_boundaryWidth = rightBoundary - leftBoundary;
		}

		public function move() : void 
		{
			x += vx;
			y += vy;
			
			// TODO re-introduce elasticity if we want to make better comparison test with performance
			// tests on other platforms
			
			// Walls collisons
			if (x < leftBoundary ) 
			{
				//vx += (x - walls.left)*elastity;
				vx = -vx;
				x = leftBoundary;
			} 
			else if (x > rightBoundary ) 
			{
				//vx += (x - walls.rightWall + d1)*elastity;
				vx = -vx;
				x = rightBoundary;
			}
			
			if (y < topBoundary ) 
			{
				//vy += (y - walls.top)*elastity;
				vy = -vy;
				y = topBoundary;
			}
			else if (y > bottomBoundary ) 
			{
				//vy += (y - walls.bottomWall + d1)*elastity;
				vy = -vy;
				y = bottomBoundary;
			}
			
			// Removed to avoid movement towards upper left
			// Locked location on integer pixel locations
			x = Math.round(x);
			y - Math.round(y);
			//			x = int(x);
			//			y = int(y);
			//			
			//			// balls seem to snap to upper-left. give them new random position if they do
			//			if (x == leftBoundary && y == topBoundary)
			//			{
			//				resetPosition();
			//				//trace(this.name,_leftBoundary,_topBoundary);
			//			}
		}
		
		public function doCollide (b : FlashBall) : Boolean 
		{
			// Calculate vectors 
			var dx : Number = x - b.x;
			var dy : Number = y - b.y;
			
			if (   (dx ^ (dx >> 31)) - (dx >> 31) > diameter || (dy ^ (dy >> 31)) - (dy >> 31) > diameter ) 
			{
				return false;
			}

			var distanceSquared : Number = dx * dx + dy * dy;
		
			if (distanceSquared > diamSquared)
			{
				return false;
			}
			var dvx : Number = vx - b.vx;
			var dvy : Number = vy - b.vy;	
			
			var mag : Number = dvx * dx + dvy * dy;
			
			// See if balls move towards each other	
			if (mag >= 0) 
			{
				return false;
			}
			
			mag /= distanceSquared;
			
			var delta_vx : Number = dx * mag;
			var delta_vy : Number = dy * mag;
			
			vx -= delta_vx;
			vy -= delta_vy;
			
			b.vx += delta_vx;
			b.vy += delta_vy;
			
			// Apply new velocities to balls
			x += vx;
			y += vy;
			
			b.x += b.vx;
			b.y += b.vy;
				
			return true;
		}
		
		public function cleanup():void
		{
			// empty
		}
		
		/**
		 * Removes the asset from the display list
		 */
		public function remove() : void {
			try {
				if (asset) asset.parent.removeChild(asset);
			} catch (error:Error) {
				trace("ERROR: failed to find an asset to remove");
			}
		}
		
	}

}