﻿/**
 * NOTES:
 * Built by roguish.com
 * contact: info@roguish.com
 * original collision detection/reaction source courtesy of: http://bubblemark.com/
 * 
 * Testing hardware device/platform: Google Nexus One, Android 2.2.1, AIR 2.6 for mobile
 * 
 * CacheAsBitmapMatrix currently switches between null and a shared Matrix object. There's probably no need to set the 
 * CABM object to null when not using CABM because it's probably just a Matrix object that is stored as a property of the 
 * Display Object so when cacheAsBitmap is not used, the stored CABM Matrix object would probably just be ignored.  
 * 
 * v.2.0.0 
 * -- Original public release
 * 
 * v.2.0.1
 * -- Updated Boundary calculations for liquid layout.
 * -- Published for AIR 2.6 and experienced much better performance for alpha fading. 
 * 
 * v.2.0.2
 * -- Prevented app from freezing when adding balls. RE: bug with StageQuality/stage.quality where 
 * setting the stage.quality to HIGH, then processing some display objects, and finally setting the stage.quality back 
 * to LOW (all within the same method) the player would consistently freeze on the Nexus One. Reported bug to Adobe. 
 * Related notes: There is a known mismatch in the case of the StageQuality constants. i.e., StageQuality.LOW/MEDIUM/HIGH/BEST
 * all return lower-case values. See the comments at the bottom of this livedocs page: 
 * http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/flash/display/StageQuality.html
 * Livedocs also reveal that the stage.quality in AIR can only be set to HIGH or BEST:
 * http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/flash/display/Stage.html#quality
 * Attempting to set stage.quality to LOW or MEDIUM does not modify the prior stage.quality value. It's not clear whether this holds true 
 * in AIR for mobile. When I checked the stage.quality setting for a SWF published to a device 
 * I saw values of "low" or "medium" when those settings are chosen. 
 * http://forums.adobe.com/thread/427899
 * 
 * v.2.0.3
 * -- Added radio group for setting stage quality since it apparently makes a significant difference. 
 * -- Moved the frame script calculation end back to the bottom of the ENTER_FRAME so it will include the blitting rendering
 * -- Added logo
 *
 * v.2.0.4 
 * -- Created a Point object to reference on each ENTER_FRAME of blitting instead of creating a new Point each ENTER_FRAME
 * -- Discovered that locking ball locations to ints was the reason balls favored the upper-left corner and removed the integer locking. 
 * 			This fix also saved many int calls as a result and may account for some of the performance gain in this version. 
 * 			I may try rounding the value up/down to an int in future versions because placing art on fixed pixel values has always 
 * 			been a recommended best practice in Flash, although mobile devices with high screen resolutions may negate that old advice.
 * -- Converted the FlashBall class to an Object rather than a Sprite, injecting a reference to a Sprite when displaying the ball as a Display Object. 
 * 			This caused significant frame script execution performance gains (now down to ~1-4ms/frame), and is also the likely cause of 
 * 			the framerate performance gains as well.
 * -- Display separate code execution times for physics frame script execution time and full script calculation time  
 * -- Applied velocities when collisions occur rather than waiting until the next frame. This was not a performance optimization, 
 * 			it just seems like that was missing in the original collision code.
 * -- Added option to inline the move and doCollide method calls.
 * -- Added option for opaque backgrounds for Display Objects
 * -- Added stage.stageWidth = Capabilities.screenResolutionX; per blog post: http://blogs.adobe.com/actionscriptdocs/category/air-releases
 * -- Show/Hide balls made a single toggle button
 * 
 * TODO: 
 * -- Add CheckBox to lock position to integer values with fast Math.round
 * -- Add rotation and alpha to blitting mode
 * -- Add weblink to logo
 * -- Code cleanup
 */


package 
{
	import fl.controls.*;
	
	import flash.desktop.NativeApplication;
	import flash.display.*;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.geom.*;
	import flash.geom.Matrix;
	import flash.system.Capabilities;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFieldType;
	import flash.text.TextFormat;
	import flash.utils.getTimer;

	public class FlashBalls_PerformanceTester extends Sprite
	{
		// label 
		public static const VERSION:String = "2.0.4";
		public static const VERSION_LABEL:String = "";
		
		// consts
		public static const WALL_LEFT:int = 0;
		public static const WALL_TOP:int = 140;
		public static const WALL_RIGHT:int = 0;
		public static const WALL_BOTTOM:int = 0;
		
		public static const NUM_BALLS_INITIAL:int = 130;
		public static const FRAME_RATE:int = 120;
		public static const TARGET_FPS:int = 20;
		public static const FPS_TICKRATE:uint = 30;
		public static const SHARED_BITMAP_MATRIX:Matrix = new Matrix();
		public static const PARTICLE_WIDTH:int = 52;
		public static const PARTICLE_HEIGHT:int = 52;
		public static const HALF_PARTICLE_WIDTH:int = 26;
		public static const HALF_PARTICLE_HEIGHT:int = 26;
		public static const ROTATION_INCREMENT_DEGREES:int = 5;
		public static const DISPLAY_MODE_DISPLAY_OBJECT:String = "DisplayObject";
		public static const DISPLAY_MODE_BLITTING:String = "Blitting";
		public static const RASTER:String = "Raster";
		public static const VECTOR:String = "Vector";
		public static const BLITTING:String = "Blitting";
		public static const STAGE_QUALITY_LOW:String = "LOW";
		public static const STAGE_QUALITY_MEDIUM:String = "MEDIUM";
		public static const STAGE_QUALITY_HIGH:String = "HIGH";
		public static const STAGE_QUALITY_BEST:String = "BEST";
		
		private var _displayMode:String = DISPLAY_MODE_DISPLAY_OBJECT;
		private var _cacheAsBitmap:Boolean = false;
		private var _cacheAsBitmapMatrix:Boolean = false;
		private var _blitting:Boolean = false;
		private var _alphaFade:Boolean = false;
		private var _rotateObjs:Boolean = false; 
		
		// UI
		private var _displayUI:fl.controls.CheckBox;
		private var _useCacheAsBitmap:fl.controls.CheckBox;
		private var _useCacheAsBitmapMatrix:fl.controls.CheckBox;
		private var _useRaster:fl.controls.RadioButton;
		private var _useVector:fl.controls.RadioButton;
		private var _useBlitting:fl.controls.RadioButton;
		private var _useAlphaFade:fl.controls.CheckBox;
		private var _useRotateObjs:fl.controls.CheckBox;
				
		// vars
		private var _n : Number; 
		private var _balls:Vector.<FlashBall> = new Vector.<FlashBall>(); 
		private var _bitmapElements:Vector.<DisplayObject> = new Vector.<DisplayObject>();
		private var _persistentForegroundElements:Vector.<DisplayObject> = new Vector.<DisplayObject>();
		private var _hideableForegroundElements:Vector.<DisplayObject> = new Vector.<DisplayObject>();
		private var _isRunning : Boolean;
		private var _f : Number = 0;
		private var _lastTime : Number;
		private var _hideBallsButton:Sprite;
		private var _showBallsButton:Sprite;
		private var _ballsCount:int = 0;
		private var _tfDisplay:TextField;
		private var _displayTextFormat:TextFormat;
		private var _ballsDOVisible:Boolean = false;
		private var _buttonAdd:Sprite;
		private var _buttonRemove:Sprite;
		private var _numBallsToAdd:int = 5;
		private var _numBallsToRemove:int = 5;
		private var _clearRectangle:Rectangle = new Rectangle();
		private var _particleRectangle:Rectangle;
		private var _sourceData:BitmapData;
        private var _canvas:BitmapData;
		private var _canvasContainer:Sprite;
		private var _particleBitmapData:BitmapData;
		private var _ballsBlittingVisible:Boolean = false;
		private var _displayBalls:Boolean = true;
		private var _textFormat:TextFormat;
		private var _textFormat2:TextFormat;
		private var _defaultFlashBallDefinition:Object;
		private var _drawMethod:String;
		private var _drawMethodRBG:RadioButtonGroup;
		private var _codeExecutionTime:int;
		private var _physicsCalcsCodeExecutionTime:int;
		private var _canvasBitmap:Bitmap;
		private var _stageQuality:String;
		private var _stageQualityRBG:RadioButtonGroup;
		private var _stageQualityLow:RadioButton;
		private var _stageQualityMedium:RadioButton;
		private var _stageQualityHigh:RadioButton;
		private var _stageQualityBest:RadioButton;
		private var _logo:Sprite;
		private var _blitPoint:Point;
		private var _inlinePhysics:Boolean;
		private var _useInlinePhysics:CheckBox;
		private var _collisionEnabled:Boolean;
		private var _useCollisionEnabled:CheckBox;
		private var _opaqueBG:Boolean;
		private var _useOpaqueBG:CheckBox;
		
		// debug
		// private var _tfOutput:TextField;
		
		function FlashBalls_PerformanceTester() {
			init(NUM_BALLS_INITIAL);
		}

		private function init($n:int):void {
			
			// debug
			// _tfOutput = new TextField();
			// _tfOutput.text = stage.quality;
			// addChild(_tfOutput);
			
			// Settings
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			stage.stageWidth = Capabilities.screenResolutionX;
			stage.stageHeight = Capabilities.screenResolutionY;
			// stage.displayState = StageDisplayState.FULL_SCREEN_INTERACTIVE;
			stage.frameRate = FRAME_RATE;
			_stageQuality = STAGE_QUALITY_LOW;
			
			_defaultFlashBallDefinition = new Object();
			_defaultFlashBallDefinition.radius = HALF_PARTICLE_WIDTH;
			_defaultFlashBallDefinition.leftWall = WALL_LEFT;
			_defaultFlashBallDefinition.topWall = WALL_TOP;
			_defaultFlashBallDefinition.rightWall = WALL_RIGHT;
			_defaultFlashBallDefinition.bottomWall = WALL_BOTTOM;
			_defaultFlashBallDefinition.stageWidth = stage.stageWidth;
			_defaultFlashBallDefinition.stageHeight = stage.stageHeight;
			
			_inlinePhysics = false;
			_collisionEnabled = true;
			_opaqueBG = false;
			
			_n = $n; 
			_blitPoint = new Point();
			_isRunning = true;
			_codeExecutionTime = 0;

			// Shared TextFormats
			_textFormat = new TextFormat(); 
			_textFormat.size = 20; 
			_textFormat2 = new TextFormat(); 
			_textFormat2.size = 30; 
			_displayTextFormat = new TextFormat();
			_displayTextFormat.size = 20;
			_displayTextFormat.font = "_sans";
			
			/// Display UI CheckBox
			_displayUI = new fl.controls.CheckBox();
			_displayUI.setStyle("textFormat", _textFormat);
            _displayUI.label = "UI";
            _displayUI.move(10, 20);
			_displayUI.selected = true;
			_displayUI.width = 500;
			// caching and gpu-acceleration settings
			_displayUI.cacheAsBitmap = true;
			_displayUI.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX
			_bitmapElements.push(_displayUI);
			_persistentForegroundElements.push(_displayUI);
			addChild(_displayUI);
			_displayUI.addEventListener(Event.CHANGE, displayUICBChangeHandler);
			
			// Display Text Field
			_tfDisplay = new TextField();
			_tfDisplay.defaultTextFormat = _displayTextFormat;
			_tfDisplay.backgroundColor = 0xFFFFFF;
			_tfDisplay.type = TextFieldType.DYNAMIC;
			_tfDisplay.x = 175;
			_tfDisplay.y = 10;
			_tfDisplay.width = 900;			
			// caching and gpu-acceleration settings
			_tfDisplay.cacheAsBitmap = true;
			_tfDisplay.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_tfDisplay);
			_persistentForegroundElements.push(_tfDisplay);
			_tfDisplay.mouseEnabled = false;
			addChild(_tfDisplay);
			
			// Logo
			_logo = new Logo();
			_logo.cacheAsBitmap = true;
			_logo.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_logo);
			_hideableForegroundElements.push(_logo);
			addChild(_logo);
			setLogoPosition();
			
			
			/**
			 * Draw Method Radio Button Group
			 */
			
			_drawMethod = RASTER;
			_drawMethodRBG = new RadioButtonGroup("rasterOrVector");
			
			// Raster
			_useRaster = new fl.controls.RadioButton();
			_useRaster.setStyle("textFormat", _textFormat);
			_useRaster.label = "Raster (Green)";
			_useRaster.group = _drawMethodRBG;
			_useRaster.move(10, 65);
			if (_drawMethod == RASTER) {
				_useRaster.selected = true;
			}
			_useRaster.width = 350;
			// caching and gpu-acceleration settings
			_useRaster.cacheAsBitmap = true;
			_useRaster.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_useRaster);
			_hideableForegroundElements.push(_useRaster);
			addChild(_useRaster);
			
			// Vector
			_useVector = new fl.controls.RadioButton();
			_useVector.setStyle("textFormat", _textFormat);
			_useVector.label = "Vector (Red)";
			_useVector.group = _drawMethodRBG;
			_useVector.move(200, 65);
			if (_drawMethod == VECTOR) {
				_useVector.selected = true;
			}
			_useVector.width = 350;
			// caching and gpu-acceleration settings
			_useVector.cacheAsBitmap = true;
			_useVector.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_useVector);
			_hideableForegroundElements.push(_useVector);
			addChild(_useVector);
			
			// Blitting
			_useBlitting = new fl.controls.RadioButton();
			_useBlitting.setStyle("textFormat", _textFormat);
			_useBlitting.label = "Blitting (Purple)";
			_useBlitting.move(390, 65);
			_useBlitting.group = _drawMethodRBG;
			if (_drawMethod == BLITTING) {
				_useBlitting.selected = true;
			}
			_useBlitting.width = 350;
			// caching and gpu-acceleration settings
			_useBlitting.cacheAsBitmap = true; 
			_useBlitting.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_useBlitting);
			_hideableForegroundElements.push(_useBlitting);
			addChild(_useBlitting);
			
			_drawMethodRBG.addEventListener(Event.CHANGE, drawModeRBGChangeHandler);
			
			/**
			 * Stage Quality Radio Button Group
			 */
			
			_stageQualityRBG = new RadioButtonGroup("stageQualityRBG");
			
			// Low
			_stageQualityLow = new fl.controls.RadioButton();
			_stageQualityLow.setStyle("textFormat", _textFormat);
			_stageQualityLow.label = "LOW";
			_stageQualityLow.group = _stageQualityRBG;
			_stageQualityLow.move(10, 100);
			if (_stageQuality == STAGE_QUALITY_LOW)	{
				_stageQualityLow.selected = true;
			}
			_stageQualityLow.width = 200;
			// caching and gpu-acceleration settings
			_stageQualityLow.cacheAsBitmap = true;
			_stageQualityLow.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_stageQualityLow);
			_hideableForegroundElements.push(_stageQualityLow);
			addChild(_stageQualityLow);
			
			// Medium
			_stageQualityMedium = new fl.controls.RadioButton();
			_stageQualityMedium.setStyle("textFormat", _textFormat);
			_stageQualityMedium.label = "MED";
			_stageQualityMedium.group = _stageQualityRBG;
			_stageQualityMedium.move(110, 100);
			if (_stageQuality == STAGE_QUALITY_MEDIUM) {
				_stageQualityMedium.selected = true;
			}
			_stageQualityMedium.width = 200;
			// caching and gpu-acceleration settings
			_stageQualityMedium.cacheAsBitmap = true;
			_stageQualityMedium.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_stageQualityMedium);
			_hideableForegroundElements.push(_stageQualityMedium);
			addChild(_stageQualityMedium);
			
			// High
			_stageQualityHigh = new fl.controls.RadioButton();
			_stageQualityHigh.setStyle("textFormat", _textFormat);
			_stageQualityHigh.label = "HIGH";
			_stageQualityHigh.group = _stageQualityRBG;
			_stageQualityHigh.move(210, 100);
			if (_stageQuality == STAGE_QUALITY_HIGH) {
				_stageQualityHigh.selected = true;
			}
			_stageQualityHigh.width = 200;
			// caching and gpu-acceleration settings
			_stageQualityHigh.cacheAsBitmap = true;
			_stageQualityHigh.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_stageQualityHigh);
			_hideableForegroundElements.push(_stageQualityHigh);
			addChild(_stageQualityHigh);
			
			// Best
			_stageQualityBest = new fl.controls.RadioButton();
			_stageQualityBest.setStyle("textFormat", _textFormat);
			_stageQualityBest.label = "BEST";
			_stageQualityBest.group = _stageQualityRBG;
			_stageQualityBest.move(310, 100);
			if (_stageQuality == STAGE_QUALITY_HIGH) {
				_stageQualityBest.selected = true;
			}
			_stageQualityBest.width = 200;
			// caching and gpu-acceleration settings
			_stageQualityBest.cacheAsBitmap = true;
			_stageQualityBest.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_stageQualityBest);
			_hideableForegroundElements.push(_stageQualityBest);
			addChild(_stageQualityBest);
			
			_stageQualityRBG.addEventListener(Event.CHANGE, stageQualityRBGChangeHandler);
			
			// bug
			// trace(StageQuality.LOW, StageQuality.MEDIUM, StageQuality.HIGH, StageQuality.BEST);
			// outputs: low medium high best
			
			
			/**
			 * Show/Hide Balls
			*/
			
			// Show Balls
			_showBallsButton = new ShowBallsButton();
			_showBallsButton.x = 10;
			_showBallsButton.y = 150;
			// caching and gpu-acceleration settings
			_showBallsButton.cacheAsBitmap = true;
			_showBallsButton.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_showBallsButton);
			_hideableForegroundElements.push(_showBallsButton);
			addChild(_showBallsButton);
			_showBallsButton.addEventListener(MouseEvent.MOUSE_DOWN,showBallsMouseEventHandler);
			_showBallsButton.visible = false;
			
			// Hide Balls
			_hideBallsButton = new HideBallsButton();
			_hideBallsButton.x = 10;
			_hideBallsButton.y = 150;
			// caching and gpu-acceleration settings
			_hideBallsButton.cacheAsBitmap = true;
			_hideBallsButton.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_hideBallsButton);
			_hideableForegroundElements.push(_hideBallsButton);
			addChild(_hideBallsButton);
			_hideBallsButton.addEventListener(MouseEvent.MOUSE_DOWN,hideBallsMouseEventHandler);
			_hideBallsButton.visible = true;
			
			// Cache As Bitmap checkbox
			_useCacheAsBitmap = new fl.controls.CheckBox();
			_useCacheAsBitmap.setStyle("textFormat", _textFormat);
            _useCacheAsBitmap.label = "CacheAsBitmap";
            _useCacheAsBitmap.move(10, 250);
			_useCacheAsBitmap.selected = _cacheAsBitmap;
			_useCacheAsBitmap.width = 600;
			// caching and gpu-acceleration settings
			_useCacheAsBitmap.cacheAsBitmap = true;
			_useCacheAsBitmap.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_useCacheAsBitmap);
			_hideableForegroundElements.push(_useCacheAsBitmap);
			addChild(_useCacheAsBitmap);
			_useCacheAsBitmap.addEventListener(Event.CHANGE, useCacheAsBitmapCBChangeHandler);
			
			// Cache As Bitmap Matrix checkbox
			_useCacheAsBitmapMatrix = new fl.controls.CheckBox();
			_useCacheAsBitmapMatrix.setStyle("textFormat", _textFormat);
            _useCacheAsBitmapMatrix.label = "CacheAsBitmapMatrix";
            _useCacheAsBitmapMatrix.move(10, 300);
			_useCacheAsBitmapMatrix.selected = _cacheAsBitmapMatrix;
			_useCacheAsBitmapMatrix.width = 600;
			// caching and gpu-acceleration settings
			_useCacheAsBitmapMatrix.cacheAsBitmap = true;
			_useCacheAsBitmapMatrix.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_useCacheAsBitmapMatrix);
			_hideableForegroundElements.push(_useCacheAsBitmapMatrix);
			addChild(_useCacheAsBitmapMatrix);
			_useCacheAsBitmapMatrix.addEventListener(Event.CHANGE, useCacheAsBitmapMatrixCBChangeHandler);
			
			// Alpha checkbox
			_useAlphaFade = new fl.controls.CheckBox();
			_useAlphaFade.setStyle("textFormat", _textFormat);
            _useAlphaFade.label = "Alpha";
            _useAlphaFade.move(10, 350);
			_useAlphaFade.selected = _alphaFade;
			_useAlphaFade.width = 600;
			// caching and gpu-acceleration settings
			_useAlphaFade.cacheAsBitmap = true;
			_useAlphaFade.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_useAlphaFade);
			_hideableForegroundElements.push(_useAlphaFade);
			addChild(_useAlphaFade);
			_useAlphaFade.addEventListener(Event.CHANGE, useAlphaFadeCBChangeHandler);

			// Rotation checkbox
			_useRotateObjs = new fl.controls.CheckBox();
			_useRotateObjs.setStyle("textFormat", _textFormat);
            _useRotateObjs.label = "Rotate";
            _useRotateObjs.move(10, 400);
			_useRotateObjs.selected = _rotateObjs;
			_useRotateObjs.width = 350;
			// caching and gpu-acceleration settings
			_useRotateObjs.cacheAsBitmap = true;
			_useRotateObjs.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_useRotateObjs);
			_hideableForegroundElements.push(_useRotateObjs);
			addChild(_useRotateObjs);
			_useRotateObjs.addEventListener(Event.CHANGE, useRotateObjsCBChangeHandler);
			
			// Inline Physics checkbox
			_useInlinePhysics = new fl.controls.CheckBox();
			_useInlinePhysics.setStyle("textFormat", _textFormat);
			_useInlinePhysics.label = "Inline Physics";
			_useInlinePhysics.move(360, 250);
			_useInlinePhysics.selected = _inlinePhysics;
			_useInlinePhysics.width = 350;
			// caching and gpu-acceleration settings
			_useInlinePhysics.cacheAsBitmap = true;
			_useInlinePhysics.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_useInlinePhysics);
			_hideableForegroundElements.push(_useInlinePhysics);
			addChild(_useInlinePhysics);
			_useInlinePhysics.addEventListener(Event.CHANGE, useInlinePhysicsCBChangeHandler);
			
			// Collision Enabled checkbox
			_useCollisionEnabled = new fl.controls.CheckBox();
			_useCollisionEnabled.setStyle("textFormat", _textFormat);
			_useCollisionEnabled.label = "Ball Collisions";
			_useCollisionEnabled.move(360, 300);
			_useCollisionEnabled.selected = _collisionEnabled;
			_useCollisionEnabled.width = 350;
			// caching and gpu-acceleration settings
			_useCollisionEnabled.cacheAsBitmap = true;
			_useCollisionEnabled.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_useCollisionEnabled);
			_hideableForegroundElements.push(_useCollisionEnabled);
			addChild(_useCollisionEnabled);
			_useCollisionEnabled.addEventListener(Event.CHANGE, useCollisionEnabledCBChangeHandler);
			
			// Opaque Background checkbox
			_useOpaqueBG = new fl.controls.CheckBox();
			_useOpaqueBG.setStyle("textFormat", _textFormat);
			_useOpaqueBG.label = "Opaque BG";
			_useOpaqueBG.move(360, 350);
			_useOpaqueBG.selected = _opaqueBG;
			_useOpaqueBG.width = 350;
			// caching and gpu-acceleration settings
			_useOpaqueBG.cacheAsBitmap = true;
			_useOpaqueBG.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_useOpaqueBG);
			_hideableForegroundElements.push(_useOpaqueBG);
			addChild(_useOpaqueBG);
			_useOpaqueBG.addEventListener(Event.CHANGE, useOpaqueBGCBChangeHandler);
			
			/**
			 * Add and Remove Balls buttons
			 */
			
			// Add Balls
			// MovieClip dimensions: 130w/115h, upper-left registration point
			_buttonAdd = new AddBallsButton();
			setAddBallsButtonPosition();
			// caching and gpu-acceleration settings
			_buttonAdd.cacheAsBitmap =  true;
			_buttonAdd.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_buttonAdd);
			_hideableForegroundElements.push(_buttonAdd);
			addChild(_buttonAdd);
			_buttonAdd.addEventListener(MouseEvent.MOUSE_DOWN,buttonAddMouseHandler);
			
			// Remove Balls
			// MovieClip dimensions: 130w/115h, upper-left registration point
			_buttonRemove = new RemoveBallsButton();
			setRemoveBallsButtonPosition();
			// caching and gpu-acceleration settings
			_buttonRemove.cacheAsBitmap =  true;
			_buttonRemove.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
			_bitmapElements.push(_buttonRemove);
			_hideableForegroundElements.push(_buttonRemove);
			addChild(_buttonRemove);
			_buttonRemove.addEventListener(MouseEvent.MOUSE_DOWN,buttonRemoveMouseHandler);
			
			/**
			 * Blitting stuff
			 */
			_particleBitmapData = new BallBitmapData(52,52);
			_particleRectangle = new Rectangle(0,0,PARTICLE_WIDTH,PARTICLE_HEIGHT);   								

			_clearRectangle.x = 0
		  	_clearRectangle.y = 0
		  	
			_canvasContainer = new Sprite();
            _canvasContainer.x = 0;
            _canvasContainer.y = 0;
            addChildAt(_canvasContainer, 0);
			
			setBlittingDimensions();
			_bitmapElements.push(_canvasBitmap);
					
			/**
			 * Final initialization stuff
			 */
			addBalls(_n);
			showBalls();
			
			_lastTime = getTimer();
			
			NativeApplication.nativeApplication.addEventListener(Event.DEACTIVATE,quitApplication);

			stage.addEventListener(Event.RESIZE,stageResizeHandler);

			addEventListener(Event.ENTER_FRAME, enterFrameHandler);
			
			trace("init complete");
		}
		
		
		
		private function enterFrameHandler(evt:Event) : void 
		{
			var tCodeExecutionTime:int = getTimer();
			var tPhysicsCalcsCodeExecutionTime:int;
			
			var tNumBalls:int = _ballsCount;
			var b1:FlashBall;
			
			var i:int = 0;
			var j:int = 0;
			
			for  (i; i<tNumBalls; i++) 
			{
				b1 = _balls[i];
				// Alpha fade
				if (_alphaFade)
				{
					b1.alpha -= .01;
					// Avoiding repeated calls to Math.random(); by simply decrementing alpha to illustrate simple alpha change
					if (b1.alpha <= .1)
					{
						b1.alpha = .9; // stay within 0-1 range to avoid alpha changes causing GPU bitmap redraw
					}
				}
				if (_rotateObjs)
				{
					b1.rotation += ROTATION_INCREMENT_DEGREES;
				}
				
				// Inlined method instead of calling it on each ball
				if (_inlinePhysics) {
					b1.x += b1.vx;
					b1.y += b1.vy;
					
					// TODO re-introduce elasticity if we want to make better comparison test with performance
					// tests on other platforms
					
					// Walls collisons
					if (b1.x < b1.leftBoundary ) 
					{
						//vx += (x - walls.left)*elastity;
						b1.vx = -b1.vx;
						b1.x = b1.leftBoundary;
					} 
					else if (b1.x > b1.rightBoundary ) 
					{
						//vx += (x - walls.rightWall + d1)*elastity;
						b1.vx = -b1.vx;
						b1.x = b1.rightBoundary;
					}
					
					if (b1.y < b1.topBoundary ) 
					{
						//vy += (y - walls.top)*elastity;
						b1.vy = -b1.vy;
						b1.y = b1.topBoundary;
					}
					else if (b1.y > b1.bottomBoundary ) 
					{
						//vy += (y - walls.bottomWall + d1)*elastity;
						b1.vy = -b1.vy;
						b1.y = b1.bottomBoundary;
					}
					
					// Good ideas gone bad... return here and create the option to lock art to integer values with 
					// an optimized Math.random method.
					// keep location on integer pixel locations
					// b1.x = Math.round(b1.x);
					// b1.y = Math.round(b1.y);
					// b1.x = int(b1.x);
					// b1.y = int(b1.y);
					// balls seem to tend towards upper-left corner. give them new random position if they do
					// possible cause for this was snapping to integer values.
					//					if (b1.x == b1.leftBoundary && b1.y == b1.topBoundary)
					//					{
					//						// this is infrequent enough that inlining it probably won't help
					//						b1.resetPosition();
					//						//trace(this.name,_leftBoundary,_topBoundary);
					//					}
				} else {
					b1.move();
				}
			}
			
			// Process collisions
			// Option to inline the doCollide method instead of calling it on each collision
			if ( _collisionEnabled ) {
				if ( _inlinePhysics ) {
					trace("using inline physics for collisions");
					// var b1:FlashBall; // declared earlier
					var b2:FlashBall;
					var dx : Number;
					var dy : Number;
					var dvx : Number;
					var dvy : Number;
					var delta_vx:Number;
					var delta_vy:Number;
					var distanceSquared:Number;
					var mag:Number;
					var diameter = b1.diameter;
					var diamSquared = b1.diamSquared;
					
					for (i=0; i<tNumBalls; i++) {
						for (j = i+1; j<tNumBalls; j++) 
						{
							b1 = _balls[i];
							b2 = _balls[j];
							
							dx = b1.x - b2.x;
							dy = b1.y - b2.y;
							
							// Testing shows that using this quick test before proceeding to distance squared shaves off a litle time...
							// ... and using the fast abs value also shaves off a little more...
							// http://lab.polygonal.de/2007/05/10/bitwise-gems-fast-integer-math/
							// sample: i = (x ^ (x >> 31)) - (x >> 31);
							// if (Math.abs(dx) > _diameter || Math.abs(dy) > _diameter)
							if ( (dx ^ (dx >> 31)) - (dx >> 31) > diameter || (dy ^ (dy >> 31)) - (dy >> 31) > diameter ) 
							{
								// Distance between the centers of both balls, in either the x or y direction,
								// is greater than the diameter of a ball, therefore the balls are not colliding, return false.
								continue;
							}
							
							// The final test to determine collision 
							// (circle-circle collision detection is simple -- distance verification) 
							distanceSquared = dx * dx + dy * dy;
							if (distanceSquared > diamSquared)
							{
								continue;
							}
							
							dvx = b1.vx - b2.vx;
							dvy = b1.vy - b2.vy;	
							
							// Make absolutely elastic collision
							mag = dvx * dx + dvy * dy;
							
							// See if balls move towards each other	
							if (mag >= 0) 
							{
								continue;
							}
							
							mag /= distanceSquared;
							
							delta_vx = dx * mag;
							delta_vy = dy * mag;
							
							b1.vx -= delta_vx;
							b1.vy -= delta_vy;
							
							b2.vx += delta_vx;
							b2.vy += delta_vy;
							
							// Apply new velocities
							b1.x += b1.vx;
							b1.y += b1.vy;
							
							b2.x += b2.vx;
							b2.y += b2.vy;
						}
					} 
				} else {
					// Non-inlined version 
					for (i=0; i<tNumBalls; i++) {
						for (j = i+1; j<tNumBalls; j++) 
						{
							_balls[i].doCollide(_balls[j]);
						}
					}
				}
			}
			
			_physicsCalcsCodeExecutionTime = getTimer() - tCodeExecutionTime;

			// Display
			if ( _displayBalls ) {
				if (_displayMode == DISPLAY_MODE_BLITTING ) {
					_canvas.lock();
					_canvas.fillRect(_clearRectangle, 0xFFFF00);
					for (i=0; i<tNumBalls; i++) {
						b1 = _balls[i];
						_blitPoint.x = b1.x - HALF_PARTICLE_WIDTH;	   			
						_blitPoint.y = b1.y - HALF_PARTICLE_HEIGHT;		
						_canvas.copyPixels(_particleBitmapData, _particleRectangle, _blitPoint, null, null, true);
					};
					_canvas.unlock();
				} else {
					for (i=0; i<tNumBalls; i++) {
						b1 = _balls[i];
						b1.asset.x = b1.x;
						b1.asset.y = b1.y;
						b1.asset.rotation = b1.rotation;
						b1.asset.alpha = b1.alpha;
					};
				}
			}

			_codeExecutionTime += getTimer() - tCodeExecutionTime;
			
			// Framerate checks at infrequent interval to avoid performance impact.
			if ( ++_f == FPS_TICKRATE ) {
				calculateFPS();
			}
			
		}
		
		/** 
		 * Adds Flash Balls to the _balls Vector.
		 * Adds the visual asset to each FlashBall
		 * Display will be handled elsewhere
		 */
		private function addBalls( pNumBalls:int ):void	{
			_ballsCount += pNumBalls;
			
			_defaultFlashBallDefinition.stageWidth = stage.stageWidth;
			_defaultFlashBallDefinition.stageHeight = stage.stageHeight;
			
			var i:int = 0;
			for (i; i<pNumBalls; i++) {
				var tNewBall:FlashBall = new FlashBall(_defaultFlashBallDefinition);
				if (_alphaFade) tNewBall.alpha = 0.1 + Math.random() * 0.8;
				if (_rotateObjs) tNewBall.rotation = int( Math.random() * 360 );
				_balls.push(tNewBall);
				addDOBallAsset(tNewBall);
			}
		}
		
		/**
		 * Removes FlashBalls instances from the _balls Vector. 
		 * 
		 * This is different than hideDOBalls and hideBlittingBalls which are used instead to
		 * just remove the visual display but keep the FlashBall in the Vector for continued processing.
		 * i.e., for Blitting and for displaying processing performance with no visual display. 
		 */
		private function removeBalls(pNumBalls:int):void
		{
			_ballsCount -= pNumBalls;
			
			var i:int = 0;
			for (i; i<pNumBalls; i++) {
				if (_balls.length == 0)
				{
					break;
				}
				var tBall:FlashBall = _balls[0];
				tBall.cleanup();
				removeElementFromBitmapElements(tBall.asset);
				// tBall.parent.removeChild(tBall);
				tBall.remove();
				tBall.asset = null;
				_balls.shift();
			}
		}
		
		private function removeElementFromBitmapElements(pElement:DisplayObject):void
		{
			var i:int = 0;
			var length:int = _bitmapElements.length;
			for (i; i<length; i++)
			{
				if (_bitmapElements[i] == pElement)
				{
					_bitmapElements.splice(i,1);
					return;
				}
			}
		}
		
		private function setBlittingDimensions():void
		{
			_clearRectangle.width = stage.stageWidth;
			_clearRectangle.height = stage.stageHeight;
			_canvas = new BitmapData(stage.stageWidth, stage.stageHeight);
			_canvasBitmap = new Bitmap(_canvas);
			while (_canvasContainer.numChildren)
			{
				_canvasContainer.removeChildAt(0);
			}
			_canvasContainer.addChild(_canvasBitmap);
		}
		
		private function setAllBallBoundaries():void
		{
			var tBall:FlashBall;
			for each (tBall in _balls)
			{
				tBall.setBoundaries(stage.stageWidth, stage.stageHeight);
			}
		}
		
		
		private function addDOBallAsset($flashBall:FlashBall):void
		{
			if ( !( _displayMode == DISPLAY_MODE_DISPLAY_OBJECT ) ) return;
			
			// stage.quality = "BEST";
			var tBallAsset:Sprite;
			
			$flashBall.remove();
			$flashBall.asset = null;
			
			// Create and prepare the new asset ball
			if (_drawMethod == RASTER)
			{
				tBallAsset = new RasterBall();
			}
			else
			{
				tBallAsset = new VectorBall();
			}
			tBallAsset.x = $flashBall.x;
			tBallAsset.y = $flashBall.y;
			if (_opaqueBG) {
				tBallAsset.opaqueBackground = 0xFFFF00;
			} 
			
			// Assign the new asset ball to the FlashBall instance
			$flashBall.asset = tBallAsset;
		
			// stage.quality = "LOW";
		}
		
		
		private function resetAllBalls():void {
			trace("resetAllBalls()");
			var tBallsCount:int = _ballsCount;
			
			removeBalls(_ballsCount);
			addBalls(tBallsCount);

			showBalls();
		};
		
		
		/*
		 * 
		 * 
		 * Display/Hide Balls
		 *
		 * 
		 */
		
		private function showBallsMouseEventHandler(evt:MouseEvent):void {
			if (_displayBalls) return; // already displaying
			
			_displayBalls = true;
			showBalls();

			_showBallsButton.visible = false;
			_showBallsButton.mouseEnabled = false;
			_hideBallsButton.visible = true;
			_hideBallsButton.mouseEnabled = true;
		}
		
		private function hideBallsMouseEventHandler( evt:MouseEvent ):void 
		{
			if (!_displayBalls) return; // already hiding balls

			_displayBalls = false;
			hideDOBalls();
			hideBlittingBalls();
			
			_showBallsButton.visible = true;
			_showBallsButton.mouseEnabled = true;
			_hideBallsButton.visible = false;	
			_hideBallsButton.mouseEnabled = false;
		}
		
		public function showBalls():void {
			trace("showBalls()");
			
			if (!_displayBalls) {
				return;
			}
			
			if (_displayMode == DISPLAY_MODE_BLITTING) {
				hideDOBalls();
				showBlittingBalls();
			} else if (_displayMode == DISPLAY_MODE_DISPLAY_OBJECT) {
				hideBlittingBalls();
				showDOBalls();
			}
		}
		
		/**
		 * Assumes the ball assets have already been added to the FlashBall instances
		 */
		private function showDOBalls():void {
			var tBall:FlashBall;
			for each(tBall in _balls) {
				// this.addChild(tBall.asset); 
				if (tBall.asset) {
					
					addChild(tBall.asset);
					
					tBall.asset.cacheAsBitmap = _cacheAsBitmap;
					if (_cacheAsBitmapMatrix) tBall.asset.cacheAsBitmapMatrix = SHARED_BITMAP_MATRIX;
					tBall.asset.mouseEnabled = false;
					_bitmapElements.push(tBall.asset);
				}
			}
			_ballsDOVisible = true;
		
			showForegroundElements();
		}
		
		private function showBlittingBalls():void
		{
			addChildAt(_canvasContainer,0);
			_ballsBlittingVisible = true;

			showForegroundElements();
		}
		
		private function hideBlittingBalls():void 
		{
			try {
				removeChild( _canvasContainer )
			} catch ( error:Error ) {
				// empty
			}; 
			
			_ballsBlittingVisible = false;
		}
		
		private function hideDOBalls():void {
			var tBall:FlashBall;
			for each ( tBall in _balls ) 
			{
				tBall.remove();
				removeElementFromBitmapElements( tBall.asset );
			}
			
			_ballsDOVisible = false;
		}
		

			
		
		
		
		/**
		 * 
		 * 
		 * UI Change Handlers
		 * 
		 * 
		 */
		
		private function buttonAddMouseHandler( evt:MouseEvent ):void
		{
			addBalls( _numBallsToAdd );
			showBalls();
			showForegroundElements();
		}
		
		private function buttonRemoveMouseHandler( evt:MouseEvent ):void
		{
			removeBalls( _numBallsToRemove );
			
			if (_ballsDOVisible) showDOBalls();
			showForegroundElements();
		}
		
		private function displayUICBChangeHandler( event:Event ):void {
			showForegroundElements();
		}
		
		private function showForegroundElements():void{
			// calling addChild on these foreground elements to bring them to the top of the stack
			var tObject:DisplayObject;
			if ( _displayUI.selected )
			{
				for each (tObject in _hideableForegroundElements) {
					addChild( tObject );
				}
				for each ( tObject in _persistentForegroundElements ) {
					addChild( tObject );
				}
			} else {
				for each ( tObject in _hideableForegroundElements )	{
					removeChild( tObject );
				}
			}
		}
			
		private function useCacheAsBitmapCBChangeHandler( event:Event ):void {
			_cacheAsBitmap = _useCacheAsBitmap.selected;
			
			resetAllBalls();
		}
		
		
		private function useCacheAsBitmapMatrixCBChangeHandler(event:Event):void {
			_cacheAsBitmapMatrix = _useCacheAsBitmapMatrix.selected;
		
			resetAllBalls();
		}
		
		private function stageQualityRBGChangeHandler(event:Event):void {
			switch( _stageQualityRBG.selection )
			{
				case _stageQualityLow:
					_stageQuality = STAGE_QUALITY_LOW;
					break;
				case _stageQualityMedium:
					_stageQuality = STAGE_QUALITY_MEDIUM;
					break;
				case _stageQualityHigh:
					_stageQuality = STAGE_QUALITY_HIGH;
					break;
				case _stageQualityBest:
					_stageQuality = STAGE_QUALITY_BEST;
					break;
				default: 
					break;
			}
			stage.quality = _stageQuality;
			trace("Set stage quality by radio button: " + _stageQualityRBG.selection + ", result stage.quality: " + stage.quality);
			// debug
			// _tfOutput.text = stage.quality;
			// addChild(_tfOutput);
		}
		
		private function drawModeRBGChangeHandler(event:Event):void	{
			trace("drawModeRBGChangeHandler(), _drawMethodRBG.selection: " + _drawMethodRBG.selection);
			switch(_drawMethodRBG.selection)
			{
				case _useRaster:
					_drawMethod = RASTER;
					_displayMode = DISPLAY_MODE_DISPLAY_OBJECT;
					break;
				case _useVector:
					_drawMethod = VECTOR;
					_displayMode = DISPLAY_MODE_DISPLAY_OBJECT;
					break;
				case _useBlitting:
					_drawMethod = BLITTING;
					_displayMode = DISPLAY_MODE_BLITTING;
				default:
					break;
			} 
			
			resetAllBalls();
		}
		
		private function useAlphaFadeCBChangeHandler(event:Event):void
		{
			_alphaFade = _useAlphaFade.selected;
			
			resetAllBalls();
		}
		
		private function useRotateObjsCBChangeHandler(event:Event):void
		{
			_rotateObjs = _useRotateObjs.selected;
			
			resetAllBalls();
		}
		
		private function useInlinePhysicsCBChangeHandler(event:Event):void
		{
			_inlinePhysics = _useInlinePhysics.selected;
			trace("_inlinePhysics set: " + _inlinePhysics)
		}
		
		private function useCollisionEnabledCBChangeHandler(event:Event):void
		{
			_collisionEnabled = _useCollisionEnabled.selected;
			trace("_collisionEnabled set: " + _collisionEnabled)
		}
		
		private function useOpaqueBGCBChangeHandler(event:Event):void
		{
			_opaqueBG = _useOpaqueBG.selected;
			
			resetAllBalls();
		}
		
		
		
		/**
		 * 
		 * Liquid Layout
		 * 
		 */
		
		private function stageResizeHandler(event:Event):void
		{
			setAllBallBoundaries();
			setBlittingDimensions();
			setLogoPosition();
			setAddBallsButtonPosition();
			setRemoveBallsButtonPosition();
		}
		
		private function setLogoPosition():void
		{
			_logo.x = stage.stageWidth;
			_logo.y = 0;
		}
		
		private function setAddBallsButtonPosition():void
		{
			_buttonAdd.x = stage.stageWidth - 155;
			_buttonAdd.y = 145;
		};
		
		private function setRemoveBallsButtonPosition():void
		{
			_buttonRemove.x = stage.stageWidth - 155;
			_buttonRemove.y = 295;
		};
		


		/**
		 * 
		 * Utility
		 * 
		 */
		
		private function calculateFPS() : void 
		{
			var delta_t : Number = getTimer() - _lastTime;
			var fps : Number = (FPS_TICKRATE/delta_t)*1000;
			_tfDisplay.text = "FlashBalls v."+ VERSION + VERSION_LABEL + "      Balls: " + _ballsCount + ", FPS: " + int(fps) + "\nPhysics code: " + int( _physicsCalcsCodeExecutionTime/FPS_TICKRATE ) + "ms., All code: " + int( _codeExecutionTime/FPS_TICKRATE ) + " ms.";
			// Reset framerate counter variable
			_f = 0;
			_codeExecutionTime = 0;
			_lastTime = getTimer();
		}
		
		private function quitApplication(event:Event):void 
		{
			NativeApplication.nativeApplication.exit();	
		}
	}
}